<?php
/**
 * @author Acacia Dynamic
 * @copyright 2024 Acacia Dynamic
 * @license private
 */
// ini_set('display_errors', 1);
// ini_set('display_startup_errors', 1);
// error_reporting(E_ALL);

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once __DIR__ . '/wooInterface/WooInterface.php';
require_once __DIR__ . '/ds4woo/Kernel/Translations/DigitalSignageTranslate.php';
require_once __DIR__ . '/ds4woo/Classes/Wp_caller.php';
require_once __DIR__ . '/ds4woo/Classes/Tools_DS4.php';
require_once __DIR__ . '/ds4woo/controllers/override/UpdateVideoHandleManager.php';
require_once __DIR__ . '/ds4woo/controllers/DigitalSignageController.php';
require_once __DIR__ . '/ds4woo/controllers/PresentationController.php';
require_once __DIR__ . '/ds4woo/controllers/PlaylistsController.php';
require_once __DIR__ . '/ds4woo/controllers/VideoController.php';
require_once __DIR__ . '/ds4woo/controllers/GabaritController.php';
require_once __DIR__ . '/ds4woo/controllers/ParametreController.php';
require_once __DIR__ . '/ds4woo/Kernel/API/Manager/DigitalSignageException.php';
require_once __DIR__ . '/controllers/agents/Shortcode.php';
require_once __DIR__ . '/ds4woo/Kernel/API/Manager/UpdateManager.php';

class digitalsignage extends Module
{
    private static $moduleTabs = [];

    private static $userAliveAlreadyChecked = false;

    public function __construct()
    {
        $this->user_connection_alive();

        DigitalSignageTranslate::setLang(substr(get_bloginfo('language'), 0, 2));

        self::$moduleTabs = [
            ['parent_class' => 'SELL', 'class_name' => 'Parent', 'name' => DigitalSignageTranslate::getTraduction('Digital_Signage_4'), 'icon' => 'play_circle_filled', 'visible' => true],
            ['parent_class' => 'Parent', 'class_name' => 'Accueil', 'name' => DigitalSignageTranslate::getTraduction('Accueil'), 'icon' => '', 'visible' => true],
            ['parent_class' => 'Parent', 'class_name' => 'Videos', 'name' => DigitalSignageTranslate::getTraduction('Videos'), 'icon' => '', 'visible' => true],
            ['parent_class' => 'Parent', 'class_name' => 'Ds4Playlists', 'name' => DigitalSignageTranslate::getTraduction('Playlists'), 'icon' => '', 'visible' => true],
            ['parent_class' => 'Parent', 'class_name' => 'GabaritEdit', 'name' => DigitalSignageTranslate::getTraduction('Modeles'), 'icon' => '', 'visible' => true],
            ['parent_class' => 'Parent', 'class_name' => 'Reglages', 'name' => DigitalSignageTranslate::getTraduction('Reglages'), 'icon' => '', 'visible' => true],
        ];

        $this->name = 'digitalsignage';
        $this->tab = 'front_office_features';
        $this->version = '1.0.0';
        $this->author = 'Acacia Dynamic';
        $this->need_instance = 1;

        $this->ps_versions_compliancy = [
            'min' => '1.6',
            'max' => _PS_VERSION_,
        ];

        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = DigitalSignageTranslate::getTraduction('Digital_Signage_4_module_name');
        $this->description = DigitalSignageTranslate::getTraduction('Digital_Signage_4_description');

        $this->confirmUninstall = DigitalSignageTranslate::getTraduction('Digital_Signage_4_confim_uninstall');

        if (!Configuration::get('DigitalSignage4_name')) {
            $this->warning = DigitalSignageTranslate::getTraduction('Digital_Signage_4_no_name');
        }
    }

    public function install()
    {
        if (!parent::install()
            || !Configuration::updateValue('DigitalSignage4_name', 'DigitalSignage4')
        ) {
            return false;
        }

        return true;
    }

    /**
     * function de désinstallation de notre module :
     * - lance la désinstallation de Module parent
     * - supprime le nom du module en abse de données
     *
     * @return bool
     */
    public function uninstall()
    {
        if (!parent::uninstall()
            || !Configuration::deleteByName('DigitalSignage4_name')
        ) {
            return false;
        }

        // Supprimer le dossier vidéos
        // rrmdir(__DIR__ . "/ds4woo/Videos");
        return true;
    }

    private function rrmdir($src)
    {
        $dir = opendir($src);
        while (false !== ($file = readdir($dir))) {
            if (($file != '.') && ($file != '..')) {
                $full = $src . '/' . $file;
                if (is_dir($full)) {
                    rrmdir($full);
                } else {
                    unlink($full);
                }
            }
        }
        closedir($dir);
        rmdir($src);
    }

    public function disable($force_all = false)
    {
        // desinstallation des tabs
        foreach (self::$moduleTabs as $tab) {
            if (!$this->uninstallTab($tab['class_name'])) {
                return false;
            }
        }

        $arrayVersion = explode('.', _PS_VERSION_);
        $centiVersion = $arrayVersion[0] * 10000 + $arrayVersion[1] * 100 + $arrayVersion[2];

        $returnHook1 = $this->unregisterHook('actionOutputHTMLBefore');
        if ($centiVersion > 10800 || $centiVersion < 10703) {
            $returnHook2 = $this->unregisterHook('displayAdminProductsExtra');
        } else {
            $returnHook2 = $this->unregisterHook('displayAdminProductsMainStepLeftColumnBottom');
        }
        $returnHook3 = $this->unregisterHook('displayAdminEndContent');
        $returnHook4 = $this->unregisterHook('actionAdminControllerSetMedia');

        return parent::disable($force_all)
            && $returnHook1
            && $returnHook2
            && $returnHook3
            && $returnHook4;
    }

    public function enable($force_all = false)
    {
        Wp_caller::getInstance()->checkTableExiste();
        $customDs4TableExiste = Wp_caller::$tableExiste;

        if (!$customDs4TableExiste) {
            $request = 'CREATE TABLE if NOT EXISTS `' . Wp_caller::getInstance()->get_video_table_name() . "` (
                  `id_demande` INT PRIMARY KEY NOT NULL AUTO_INCREMENT,
                  `data` text NOT NULL,
                  `AE_version` varchar(255) NOT NULL,
                  `liste_id_boxes` varchar(255) NOT NULL DEFAULT '',
                  `flag` varchar(255) NOT NULL DEFAULT 'waiting',
                  `id_thread` varchar(255) NOT NULL,
                  `id_compte` int NOT NULL,
                  `video_name` varchar(255) NOT NULL,
                  `video_comment` varchar(255) NOT NULL,
                  `date_begin` date NOT NULL,
                  `automatique` int NOT NULL DEFAULT '0',
                  `date_end` date NOT NULL,
                  `date_demande` datetime NOT NULL,
                  `gabarit` int NOT NULL,
                  `video_url` varchar(500) NOT NULL,
                  `video_hash` varchar(1000) NOT NULL DEFAULT '',
                  `product_id` varchar(500) NOT NULL DEFAULT '',
				  `youtube_data` text NOT NULL,
				  `youtube_id` varchar(255) NOT NULL DEFAULT ''
               )  ";

            $customDs4TableExiste = Db::getInstance()->execute($request);
        }

        // installation des tabs
        foreach (self::$moduleTabs as $tab) {
            $this->installTab($tab['parent_class'], $tab['class_name'], $tab['name'], $tab['icon'], $tab['visible']);
        }

        $this->setDefaultSettings();

        $arrayVersion = explode('.', _PS_VERSION_);
        $centiVersion = $arrayVersion[0] * 10000 + $arrayVersion[1] * 100 + $arrayVersion[2];

        $returnHook1 = $this->registerHook('actionOutputHTMLBefore');
        if ($centiVersion > 10800 || $centiVersion < 10703) {
            $returnHook2 = $this->registerHook('displayAdminProductsExtra');
        } else {
            $returnHook2 = $this->registerHook('displayAdminProductsMainStepLeftColumnBottom');
        }
        $returnHook3 = $this->registerHook('displayAdminEndContent');
        $returnHook4 = $this->registerHook('actionAdminControllerSetMedia');

        return parent::enable($force_all) && $customDs4TableExiste
            && $returnHook1
            && $returnHook2
            && $returnHook3
            && $returnHook4;
    }

    public function setDefaultSettings()
    {
        $wp = new Wp_caller();

        if (!$wp->get_data(Wp_caller::PLACEMENT)) {
            $wp->save_data(Wp_caller::PLACEMENT, [
                'placement_1' => 'on',
                'placement_6' => 'on',
                'placementForm' => 'Enregistrer',
            ]);
        }

        if (!$wp->get_data(Wp_caller::RESOLUTION)) {
            $wp->save_data(Wp_caller::RESOLUTION, '1280x720');
        }
    }

    public function installTab($parent_class, $class_name, $name, $icon = '', $visible = true)
    {
        if ($visible == false) {
            $parent_class = -1;
        }

        $tab = new Tab();

        $langs = Language::getLanguages(false);
        foreach ($langs as $l) {
            $tab->name[$l['id_lang']] = $name;
        }

        $tab->icon = $icon;
        $tab->class_name = $class_name;
        $tab->id_parent = (int) Tab::getIdFromClassName($parent_class);
        $tab->module = $this->name;

        return $tab->add();
    }

    private function uninstallTab($class_name)
    {
        $tabId = (int) Tab::getIdFromClassName($class_name);
        if (!$tabId) {
            return true;
        }

        $tab = new Tab($tabId);
        $tab->name = '';

        try {
            return $tab->delete();
        } catch (PrestaShopException $e) {
            $this->warning = $e->getMessage();
        }

        return false;
    }

    public function hookActionOutputHTMLBefore($params)
    {
        $params['html'] = Shortcode::replaceShortcodes($params['html']);
    }

    public function hookDisplayAdminEndContent($p)
    {
        $this->displayUpdateMessageTabs();
    }

    public function hookActionAdminControllerSetMedia($params)
    {
        $this->context->controller->addCSS(get_site_url() . 'modules/digitalsignage/views/css/styles.css');
        $this->context->controller->addJS(get_site_url() . 'modules/digitalsignage/views/js/digitalsignage.js');
        $this->context->controller->addJS(get_site_url() . 'modules/digitalsignage/views/js/script.js');
    }

    public function hookDisplayAdminProductsExtra(array $params)
    {
        return $this->hookDisplayAdminProductsMainStepLeftColumnBottom($params);
    }

    public function hookDisplayAdminProductsMainStepLeftColumnBottom($params)
    {
        $panel = '';

        try {
            refresh_local_token();

            // product
            $productObject = new Product($params['id_product']);
            $productArray = (array) $productObject;
            $productArray['status'] = ($productArray['state'] == Product::STATE_TEMP) ? 'auto-draft' : 'saved';
            $productArray['price'] = '' . Product::getPriceStatic($params['id_product']);

            // product imgs
            $productDefaultImgs = $productObject->getImages($this->context->language->id);

            $currentThis = $this;
            $productImgs = array_map(
                function ($image) use ($currentThis, $productObject) {
                    $pictureUrl = $currentThis->context->link->getImageLink($productObject->link_rewrite, $image['id_image'], '');

                    $pictureObject = new Image($image['id_image']);

                    $pictureFileEmplacement = _PS_CORE_DIR_ . DIRECTORY_SEPARATOR . 'img' . DIRECTORY_SEPARATOR . 'p' . DIRECTORY_SEPARATOR . $pictureObject->getExistingImgPath() . '.jpg';

                    return [
                        'url' => $pictureUrl,
                        'path' => $pictureFileEmplacement,
                    ];
                },
                $productDefaultImgs
            );

            $gabarits = GabaritManager::instance()->getGabarits();

            foreach ($gabarits as $gabarit_) {
                $gabarit_->initCustomChamp();
            }

            $this->smarty->assign([
                't' => new DigitalSignageTranslate(),
                'toolsDS4' => new Tools_DS4(),
                'product' => $productArray,
                'product_img' => $productImgs,
                'urlSpecialData' => Tools_DS4::getModuleUrlPresta() . 'ds4woo/Classes/CreationVideoSpecialDataProviderAjaxHandler.php?digitalSignageSpecialData=',
                'urlChangeDisplayedVideo' => Tools_DS4::getModuleUrlPresta() . 'ds4woo/Classes/DisplayedVideoProductAjaxHandler.php',
                'urlLoadingImg' => Tools_DS4::getModuleImgDirPresta() . 'loadingChamp.gif',
                'gabarits' => $gabarits,
                'urlTraitement' => Tools_DS4::getModuleUrlPresta() . 'ds4woo/controllers/override/AddVideoHandler.php',
                'urlVideos' => get_admin_url() . 'index.php?controller=Videos&token=' . Tools::getAdminTokenLite('Videos'),
                'videoPath' => Tools_DS4::getModuleVideoUrl(),
                'updateController' => Tools_DS4::getModuleUrlPresta() . 'ds4woo/controllers/override/UpdateVideoHandler.php',
                'user_connected' => ForfaitManager::instance()->getToken() !== null,
                'currentHistoriqueHtml' => $this->getHistoriqueHtml($productObject->id),
                'UpdateMessage' => $this->getUpdateMessage(),
                // 'NbrActiveVideo'		  => ForfaitManager::instance()->getForfait(ForfaitManager::instance()->getToken(), true)->getActiveVideos() - (new VideoController())->getNbrVideosActives()
            ]);

            $panel = $this->display(__FILE__, 'views/templates/hook/front/adminProductCreateVideoForm.tpl');
        } catch (Exception $e) {
            $panel = $e->getMessage();
        }

        return $panel;
    }

    private function getUpdateMessage()
    {
        $newVersion = UpdateManager::get(Tools_DS4::getModuleInformations()['Version']);

        if ($newVersion) {
            $m .= DigitalSignageTranslate::getTraduction('Update_modal', $newVersion['link']);

            return $m;
        }

        return '';
    }

    public function getHistoriqueHtml($productId)
    {
        $contentHtml = '';

        try {
            $videoForThisProduct = DemandeManager::set(Wp_caller::getInstance())->getLocalDemandes('SELECT * FROM %s WHERE `product_id` = ' . $productId);
        } catch (Exception $e) {
            $videoForThisProduct = [];
        }

        foreach ($videoForThisProduct as $video) {
            $videoHistoriqueManager = new UpdateVideoHandlerManager(true, $video->getIdDemande(), $video->getProductId());
            $contentHtml = $contentHtml . $videoHistoriqueManager->getHtmlDatas();
        }

        return $contentHtml;
    }

    private function user_connection_alive()
    {
        $forfait = new ForfaitEntity();
        $success = $forfait->fromArray(Wp_caller::getInstance()->get_data(Wp_caller::FORFAIT));

        if ($success) {
            ForfaitManager::instance()->setForfait($forfait);
        } elseif (!self::$userAliveAlreadyChecked) { // Si aucun forfait n'existe alors on créer un compte de test pour l'utilisateur
            try {
                $forfait = ForfaitManager::instance()->createForfaitTrial();

                if (!$forfait) {
                    foreach (ForfaitManager::instance()->getUserMessages() as $error) {
                        Context::getContext()->cookie->errors = $error;
                    }
                } else {
                    Wp_caller::getInstance()->save_data(Wp_caller::FORFAIT, $forfait->toArray());
                }
            } catch (DigitalSignageException $e) {
                $e->echoError();
            }
            self::$userAliveAlreadyChecked = true;
        }
    }

    private function displayUpdateMessageTabs()
    {
        $newVersion = UpdateManager::get(Tools_DS4::getModuleInformations()['Version']);

        if ($newVersion) {
            echo DigitalSignageTranslate::getTraduction('Update_modal', $newVersion['link']);
        }
    }
}
