<?php
/**
 * @author Acacia Dynamic
 * @copyright 2024 Acacia Dynamic
 * @license private
 */
require_once __DIR__ . '/../Kernel/API/Entity/DemandeEntity.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class Tools_DS4
{
    public static function getUrlRedirect($page)
    {
        return 'admin.php?page=' . $page;
    }

    public static function redirectAdmin($page)
    {
        if (self::isPrestashop()) {
            switch ($page) {
                case 'DigitalSignageSettings':
                    Tools::redirectAdmin('index.php?controller=Reglages&token=' . Tools::getAdminTokenLite('Reglages'));

                    break;

                case 'DigitalSignageHome':
                    Tools::redirectAdmin('index.php?controller=Accueil&token=' . Tools::getAdminTokenLite('Accueil'));

                    break;

                default:
                    Tools::redirect($_SERVER['REQUEST_URI']);

                    break;
            }
        } else {
            Tools::redirect(self::getUrlRedirect($page));
        }

        exit;
    }

    public static function isHttps()
    {
        return isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on';
    }

    public static function isPrestashop()
    {
        return defined('_PS_VERSION_');
    }

    public static function getModuleImgUrl()
    {
        return self::getModuleUrlPresta() . 'views/img/';
    }

    public static function reverseMagicQuote($var) // Fonction qui supprime l'effet des magic quotes
    {
        if (is_array($var)) { // Si la variable passée en argument est un array, on appelle la fonction stripslashes_r dessus
            return array_map(['Tools_DS4', 'reverseMagicQuote'], $var);
        } else { // Sinon, un simple stripslashes suffit
            return stripslashes($var);
        }
    }

    public static function getAcaciaRedirectionPageUrl($page)
    {
        $lang = substr(get_bloginfo('language'), 0, 2);
        $cms = 'prestashop';
        $siteUrl = _PS_BASE_URL_;
        $my_currency_iso_code = Currency::getDefaultCurrency()->iso_code;

        return 'https://www.acacia-dynamic.com/redirection/ds4_redirect.php?page=' . $page . '&lang=' . $lang . '&cms=' . $cms . '&website=' . $siteUrl . '&currency=' . $my_currency_iso_code;
    }

    public static function getModuleUrlPresta()
    {
        // return str_replace('/home', '/modules/digitalsignage/ds4woo/', plugin_dir_url(__FILE__));
        return plugin_dir_url(__FILE__) . 'digitalsignage/';
    }

    public static function getModuleUrl()
    {
        return str_replace('/home', '/modules/digitalsignage/', plugin_dir_url(__FILE__));
    }

    public static function getModuleVideoUrl()
    {
        return self::getModuleUrlPresta() . 'Videos/';
    }

    public static function getShopBaseUrl()
    {
        return '';
    }

    public static function getModuleImgDir()
    {
        return self::getModuleDir() . '../Files/';
    }

    public static function getModuleImgDirPresta()
    {
        return self::getModuleUrlPresta() . 'views/img/';
    }

    public static function getModuleDir()
    {
        return plugin_dir_path(__FILE__);
    }

    public static function getModuleDirPresta()
    {
        return dirname(__FILE__) . '/../';
    }

    public static function getProductsUrl()
    {
        if (self::isPrestashop()) {
            return Context::getContext()->link->getAdminLink('AdminProducts', true);
        } else {
            return get_admin_url() . 'edit.php?post_type=product';
        }
    }

    /**
     * @param DemandeEntity[] $videos
     */
    public static function convertPoductIdToLink($videos)
    {
        return array_map(
            function ($video) {
                $product = wc_get_product($video->getProductId());

                if ($product) {
                    $permalink = $product->get_permalink();
                    $video->setProductId("<a target='_blank' href='" . $permalink . "'>" . $product->get_name() . '</a>');
                }

                return $video;
            },
            $videos
        );
    }

    public static function displayError($error)
    {
        echo '<div class="error"><p>' . $error . '</p></div>';
    }

    /**
     * retourne le numero de version du module Digital Signage
     *
     * @return mixed
     */
    public static function getModuleInformations()
    {
        return get_plugin_data(__DIR__ . '/../digitalsignage.php', false, false);
    }

    public static function getWordpressBasePath()
    {
        $currentPath = $_SERVER['SCRIPT_FILENAME'];
        $pathExploded = explode('wp-content', $currentPath);

        return $pathExploded[0];
    }

    public static function getWooCommerceDevice($currency = '', $defaultCurrency = 'EUR')
    {
        if (!$currency) {
            $currency = $defaultCurrency;
            if (function_exists('get_woocommerce_currency')) {
                $currency = get_woocommerce_currency();
            }
        }

        $symbols = apply_filters('woocommerce_currency_symbols', [
            'AED' => 'د.إ',
            'AFN' => '؋',
            'ALL' => 'L',
            'AMD' => 'AMD',
            'ANG' => 'ƒ',
            'AOA' => 'Kz',
            'ARS' => '$',
            'AUD' => '$',
            'AWG' => 'ƒ',
            'AZN' => 'AZN',
            'BAM' => 'KM',
            'BBD' => '$',
            'BDT' => '৳ ',
            'BGN' => 'лв.',
            'BHD' => '.د.ب',
            'BIF' => 'Fr',
            'BMD' => '$',
            'BND' => '$',
            'BOB' => 'Bs.',
            'BRL' => 'R$',
            'BSD' => '$',
            'BTC' => '฿',
            'BTN' => 'Nu.',
            'BWP' => 'P',
            'BYR' => 'Br',
            'BZD' => '$',
            'CAD' => '$',
            'CDF' => 'Fr',
            'CHF' => 'CHF',
            'CLP' => '$',
            'CNY' => '¥',
            'COP' => '$',
            'CRC' => '₡',
            'CUC' => '$',
            'CUP' => '$',
            'CVE' => '$',
            'CZK' => 'Kč',
            'DJF' => 'Fr',
            'DKK' => 'DKK',
            'DOP' => 'RD$',
            'DZD' => 'د.ج',
            'EGP' => 'EGP',
            'ERN' => 'Nfk',
            'ETB' => 'Br',
            'EUR' => '€',
            'FJD' => '$',
            'FKP' => '£',
            'GBP' => '£',
            'GEL' => 'ლ',
            'GGP' => '£',
            'GHS' => '₵',
            'GIP' => '£',
            'GMD' => 'D',
            'GNF' => 'Fr',
            'GTQ' => 'Q',
            'GYD' => '$',
            'HKD' => '$',
            'HNL' => 'L',
            'HRK' => 'Kn',
            'HTG' => 'G',
            'HUF' => 'Ft',
            'IDR' => 'Rp',
            'ILS' => '₪',
            'IMP' => '£',
            'INR' => '₹',
            'IQD' => 'ع.د',
            'IRR' => '﷼',
            'IRT' => 'تومان',
            'ISK' => 'kr.',
            'JEP' => '£',
            'JMD' => '$',
            'JOD' => 'د.ا',
            'JPY' => '¥',
            'KES' => 'KSh',
            'KGS' => 'сом',
            'KHR' => '៛',
            'KMF' => 'Fr',
            'KPW' => '₩',
            'KRW' => '₩',
            'KWD' => 'د.ك',
            'KYD' => '$',
            'KZT' => 'KZT',
            'LAK' => '₭',
            'LBP' => 'ل.ل',
            'LKR' => 'රු',
            'LRD' => '$',
            'LSL' => 'L',
            'LYD' => 'ل.د',
            'MAD' => 'د.م.',
            'MDL' => 'MDL',
            'MGA' => 'Ar',
            'MKD' => 'ден',
            'MMK' => 'Ks',
            'MNT' => '₮',
            'MOP' => 'P',
            'MRO' => 'UM',
            'MUR' => '₨',
            'MVR' => '.ރ',
            'MWK' => 'MK',
            'MXN' => '$',
            'MYR' => 'RM',
            'MZN' => 'MT',
            'NAD' => '$',
            'NGN' => '₦',
            'NIO' => 'C$',
            'NOK' => 'kr',
            'NPR' => '₨',
            'NZD' => '$',
            'OMR' => 'ر.ع.',
            'PAB' => 'B/.',
            'PEN' => 'S/.',
            'PGK' => 'K',
            'PHP' => '₱',
            'PKR' => '₨',
            'PLN' => 'zł',
            'PRB' => 'р.',
            'PYG' => '₲',
            'QAR' => 'ر.ق',
            'RMB' => '¥',
            'RON' => 'lei',
            'RSD' => 'дин.',
            'RUB' => '₽',
            'RWF' => 'Fr',
            'SAR' => 'ر.س',
            'SBD' => '$',
            'SCR' => '₨',
            'SDG' => 'ج.س.',
            'SEK' => 'kr',
            'SGD' => '$',
            'SHP' => '£',
            'SLL' => 'Le',
            'SOS' => 'Sh',
            'SRD' => '$',
            'SSP' => '£',
            'STD' => 'Db',
            'SYP' => 'ل.س',
            'SZL' => 'L',
            'THB' => '฿',
            'TJS' => 'ЅМ',
            'TMT' => 'm',
            'TND' => 'د.ت',
            'TOP' => 'T$',
            'TRY' => '₺',
            'TTD' => '$',
            'TWD' => 'NT$',
            'TZS' => 'Sh',
            'UAH' => '₴',
            'UGX' => 'UGX',
            'USD' => '$',
            'UYU' => '$',
            'UZS' => 'UZS',
            'VEF' => 'Bs F',
            'VND' => '₫',
            'VUV' => 'Vt',
            'WST' => 'T',
            'XAF' => 'Fr',
            'XCD' => '$',
            'XOF' => 'Fr',
            'XPF' => 'Fr',
            'YER' => '﷼',
            'ZAR' => 'R',
            'ZMW' => 'ZK',
        ]);

        $currency_symbol = isset($symbols[$currency]) ? $symbols[$currency] : '';

        return apply_filters('woocommerce_currency_symbol', $currency_symbol, $currency);
    }
}
