<?php
/**
 * @author Acacia Dynamic
 * @copyright 2024 Acacia Dynamic
 * @license private
 */
require_once __DIR__ . '/../../../Kernel/API/Entity/CompteEntity.php';
require_once __DIR__ . '/../../../Kernel/API/Entity/ForfaitEntity.php';
require_once __DIR__ . '/../../../Kernel/API/Linker/API_Linker.php';
require_once __DIR__ . '/../../../Kernel/API/Linker/CMS_Linker.php';
require_once __DIR__ . '/../../../Kernel/Translations/DigitalSignageTranslate.php';
require_once 'Manager.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class ForfaitManager extends Manager
{
    // Singleton
    private static $instance;

    /**
     * @param $cms_linker
     *
     * @return ForfaitManager|null
     *
     * @throws DigitalSignageException
     */
    public static function instance()
    {
        if (self::$instance == null) {
            self::$instance = new ForfaitManager();
        }

        return self::$instance;
    }

    // Variable du singleton

    /**
     * @var ForfaitEntity|null
     */
    private $forfait;

    private $managerEntityName = 'Forfait';

    /**
     * permet de set le forfait sans faire d'appel a l'api, par exemple depuis un instance sauvegardé dans une base de données
     *
     * @param $forfait
     */
    public function setForfait($forfait)
    {
        $this->forfait = $forfait;
    }

    /**
     * Requete qui renvois le forfait sqi ile st trouvé et l'enregistre dans son singleton
     *
     * @param $Token
     *
     * @return false|ForfaitEntity false avec un mauvais token et une array avec un bon token
     */
    public function getForfait($token = null, $forceReset = false)
    {
        if ($forceReset) {
            $oldForfait = $this->forfait;
        }

        if ($this->forfait != null && !$forceReset) {
            return $this->forfait;
        }

        if ($token) {
            try {
                $forfaits = API_Linker::get_entitys($this->managerEntityName . '?token=' . $token, ForfaitEntity::class);
            } catch (DigitalSignageException $e) {
                $e->echoError();
            }

            if ($forfaits && isset($forfaits[0])) {
                $this->forfait = $forfaits[0];
                $tempCompte = new CompteEntity();
                $this->forfait->setCompte($tempCompte->hydrate($this->forfait->getCompte()));

                return $this->forfait;
            } else {
                $this->addUserMessage(DigitalSignageTranslate::getTraduction('aucun_resultat', strtolower($this->managerEntityName)));
            }
        } else {
            $this->addUserMessage(DigitalSignageTranslate::getTraduction('champ_obligatoir', 'token'));
        }

        if ($forceReset) {
            $this->forfait = $oldForfait;
        }

        return false;
    }

    public function getToken()
    {
        if ($this->forfait != null && !($this->forfait instanceof __PHP_Incomplete_Class)) {
            return $this->forfait->getToken();
        }

        return null;
    }

    public function update_compteur_youtube_max($token = null, $nb)
    {
        try {
            // file_put_contents('aaaa.txt','update_compteur_youtube_max .'."\r\n",FILE_APPEND);
            // file_put_contents('aaaa.txt',$this->managerEntityName . "?update_compteur_youtube_max=update_compteur_youtube_max&nb=" . $nb."&token=".$token."\r\n",FILE_APPEND);

            API_Linker::post($this->managerEntityName . '?update_compteur_youtube_max=update_compteur_youtube_max&nb=' . $nb . '&token=' . $token, ['a' => 'b'], '');
            // $this->addUserMessage( 'laaa!!!!!' );
        } catch (DigitalSignageException $e) {
            $this->addUserMessage($e->getMessage());
        }
    }

    public function createForfaitTrial()
    {
        try {
            $domainSite = $_SERVER['SERVER_NAME'];
            $forfait_trial = API_Linker::post($this->managerEntityName . '?trial=true&site=' . $domainSite, [], '');
            $forfait_trial_datas = $forfait_trial->getDatas();

            if ($forfait_trial_datas && isset($forfait_trial_datas[0])) {
                $this->forfait = (new ForfaitEntity())->hydrate($forfait_trial_datas[0]);
                $this->forfait->setCompte((new CompteEntity())->hydrate($this->forfait->getCompte()));

                return $this->forfait;
            }
        } catch (DigitalSignageException $e) {
            $this->addUserMessage($e->getMessage());
        }

        return false;
    }
}
