<?php
/**
 * @author Acacia Dynamic
 * @copyright 2024 Acacia Dynamic
 * @license private
 */
require_once __DIR__ . '/../Classes/Tools_DS4.php';
require_once __DIR__ . '/../Kernel/API/Entity/AutomatisationEntity.php';
require_once __DIR__ . '/../Kernel/API/Entity/PlaylistEntity.php';
require_once __DIR__ . '/../Kernel/API/Entity/LotEntity.php';
require_once __DIR__ . '/../Kernel/API/Manager/AutomatisationManager.php';
require_once __DIR__ . '/../Kernel/API/Manager/LotManager.php';
require_once __DIR__ . '/../Kernel/API/Manager/DigitalSignageException.php';
require_once __DIR__ . '/../Kernel/API/Manager/GabaritManager.php';
require_once __DIR__ . '/../Kernel/API/Manager/PlaylistManager.php';
require_once __DIR__ . '/../Kernel/Translations/DigitalSignageTranslate.php';
require_once __DIR__ . '/../Classes/Wp_caller.php';
require_once __DIR__ . '/../Kernel/API/Manager/DemandeManager.php';
require_once __DIR__ . '/../controllers/hook/AutomatisationHookControllerCreation.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class VideoController extends DigitalSignageController
{
    public function generatePage()
    {
        try {
            $this->route();
        } catch (DigitalSignageException $e) {
            $e->echoError();
        }
    }

    /**
     * @throws DigitalSignageException
     */
    private function route()
    {
        $vue = (Tools::getIsset('automatisationVue')) ? (Tools::getValue('automatisationVue')) : 'liste';
        $vue_lot = (Tools::getIsset('traitement_par_lotVue')) ? (Tools::getValue('traitement_par_lotVue')) : 'liste';

        $this->handleDeleteVideoAjaxRequest();
        $this->handleCreationAjaxRequest();

        if ($vue == 'add') {
            $this->generateCreationPage();

            return;
        }

        if ($vue_lot == 'add') {
            $this->generateCreationLotPage();

            return;
        }

        if ($vue == 'liste') {
            // Nombre de videos affichées par page de videotheque
            $videosPage = 10;

            $offset = (int) ((Tools::getIsset('offset')) ? Tools::getValue('offset') : 0);
            if ($offset > 0) {
                $offset = $offset * $videosPage;
            }

            // Récuperation des videos
            $videos = $this->getVideo($offset, $videosPage);

            $videosHtml = array_map(
                function ($v) {
                    $u = new UpdateVideoHandlerManager(false, $v->getIdDemande(), $v->getFlag());

                    return $u->getHtmlDatas();
                },
                $videos
            );

            // Total des videos
            $pagination = $this->pagination($offset, $videosPage);
            //
            //                $videos = Tools_DS4::convertPoductIdToLink($videos);

            $url = plugin_dir_url(__FILE__);

            $this->assign([
                'pageProduits' => Tools_DS4::getProductsUrl(),
                'automatisations' => $this->getAutomatisation(),
                'ManualUrl' => Tools_DS4::getModuleImgDirPresta() . 'DS4-Manual.png',
                'AutomateUrl' => Tools_DS4::getModuleImgDirPresta() . 'DS4-Automate.png',
                'BatchUrl' => Tools_DS4::getModuleImgDirPresta() . 'DS4-Batch.png',
                'VideosUrl' => Tools_DS4::getModuleImgDirPresta() . 'DS4-Videos.png',
                'traitement_par_lots' => $this->getLot(),
                'playlistBaseUrl' => Tools_DS4::getUrlRedirect('DigitalSignagePlaylists'),
                'videos' => $videosHtml,
                'puces' => $pagination,
                'stats' => $this->getStats(),
                'moduleUrl' => Tools_DS4::getModuleUrlPresta(),
                'updateController' => Tools_DS4::getModuleUrlPresta() . 'ds4woo/controllers/override/UpdateVideoHandler.php',
                'textBtnCreation' => (Tools_DS4::isPrestashop()) ? DigitalSignageTranslate::getTraduction('Creation_manuelle_btn_pre') : DigitalSignageTranslate::getTraduction('Creation_manuelle_btn_woo'),
            ]);

            $this->setWPTemplate('Videos/videotheque');
        }// else {
        // $this->generateCreationPage();
        // }
    }

    /**
     * @throws DigitalSignageException
     */
    private function getVideo($offset, $videosPage)
    {
        $sql = "SELECT * from %s ORDER BY `id_demande` DESC, CASE `flag` WHEN 'waiting' THEN 1 WHEN 'in_progress' THEN 2 WHEN 'completed' THEN 3 WHEN 'error' THEN 4 ELSE 5 END  LIMIT $videosPage OFFSET $offset";

        return DemandeManager::set(Wp_caller::getInstance())->getLocalDemandes($sql);
    }

    private function pagination($offset, $videosPage)
    {
        $nombreDeVideo = Wp_caller::getInstance()->execute_video_request("SELECT count(*) as 'cpt' FROM %s");

        if ($nombreDeVideo) {
            $nombreDeVideo = $nombreDeVideo[0]['cpt'];

            $puces = [];

            for ($i = 0; $i < $nombreDeVideo / $videosPage; ++$i) {
                $puces[] = [
                    'active' => ($offset / $videosPage == $i),
                    'nombre' => $i + 1,
                    'lien' => (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http') . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]&offset=$i",
                ];
            }

            return $puces;
        }
    }

    private function getLot()
    {
        $lots = LotManager::set(Wp_caller::getInstance())->get_lots();

        foreach ($lots as $key => $lot) {
            // Playlsit
            $playlist = PlaylistManager::set(Wp_caller::getInstance())->get_playlist($lot->getPlaylist());
            if ($playlist) {
                $lots[$key]->setPlaylist($playlist);
            } else {
                $lots[$key]->setPlaylist(null);
            }

            // Gabarit
            $gabarit = GabaritManager::instance()->getGabarit($lot->getGabarit());
            if ($gabarit) {
                $lots[$key]->setGabarit($gabarit);
            }
        }

        return $lots;
    }

    public function generateCreationLotPage()
    {
        if (Tools::getIsset('save_lot')) {
            ob_get_clean();
            ob_start();

            if (empty($_POST['when'])) {
                $_POST['when'] = ' ';
            }

            $update = true;

            $id = Tools::getValue('id');
            if (!$id) {
                $id = LotManager::set(Wp_caller::getInstance())->get_last_id() + 1;
                $update = false;
            }

            $lot = new LotEntity();
            $lot->setActive(true);
            $lot->setId($id);
            $lot->setName(stripslashes(Tools::getValue('name')));
            $lot->setCategorie((Tools::getValue('categories') == '') ? [] : explode(',', Tools::getValue('categories')));
            $lot->setCreated(0);
            // $lot->setHookCreation((strpos($_POST['when'], 'creat') !== false));
            // $lot->setHookSell((strpos($_POST['when'], 'sell') !== false));
            $lot->setPriceMax(Tools::getValue('prixMax'));
            $lot->setPriceMin(Tools::getValue('prixMin'));
            $lot->setGabarit(Tools::getValue('gabarit'));
            $lot->setPlaylist(Tools::getValue('playlist'));
            // $lot->setNbProduit($_POST['sellNbProduit']);
            // $lot->setNbJours($_POST['sellNbJours']);

            try {
                $lotInstance = new LotManager(Wp_caller::getInstance());

                if ($update) {
                    if ($lotInstance->get_lot($id) == $lot) {
                        echo DigitalSignageTranslate::getTraduction('Parametres_inchangé');
                    } else {
                        if ($lotInstance->update_lot($lot)) {
                            echo 'ok:' . $id;
                        } else {
                            echo DigitalSignageTranslate::getTraduction('MAJ_bdd_impossible');
                        }
                    }
                } else {
                    if ($lotInstance->add_lot($lot)) {
                        echo 'ok:' . $id;
                    } else {
                        echo DigitalSignageTranslate::getTraduction('MAJ_bdd_impossible');
                    }
                }
            } catch (DigitalSignageException $e) {
                echo 'erreur : ' . $e->getMessage();
            }

            // lancer la creation de toutes les vidéos...
            $automatisationsRunner = new AutomatisationHookControllerCreation();
            // lister les id de produits dont il faut créer la vidéo
            $Lot_ = LotManager::set(Wp_caller::getInstance());
            $liste_product = $Lot_->get_products(Tools::getValue('categories'), Tools::getValue('prixMin'), Tools::getValue('prixMax'));

            foreach ($liste_product as $id_product_) {
                // $wooCommerceProduct = new WC_Product($id_product_);
                $prestaProduct = [];
                $id_lang = (int) Configuration::get('PS_LANG_DEFAULT');
                $prestaProduct = new Product($id_product_, false, $id_lang);

                $datasForPostDemande = $automatisationsRunner->getDatasForPostDemandeAvecGabarit(Tools::getValue('gabarit'), $prestaProduct, null);
                $postDemandeResponse = $automatisationsRunner->postVideoToApi($datasForPostDemande, $id + 1);
                var_dump($postDemandeResponse);
            }

            exit(ob_get_clean());
        }

        $defautData = [
            'id' => '',
            'name' => '',
            'playlist' => '',
            'priceMin' => '',
            'priceMax' => '',
            'categorieTree' => $this->categorieTree(),
            'gabarit' => '',
            'nbProducts' => '5',
            'nbJours' => '5',
            'playlistVideoCount' => '0',
        ];

        if (Tools::getIsset('lot_id')) {
            $lot = LotManager::set(Wp_caller::getInstance())->get_lot(Tools::getValue('lot_id'));
            if ($lot) {
                $defautData['id'] = $lot->getId();
                $defautData['name'] = $lot->getName();
                $defautData['playlist'] = $lot->getPlaylist();
                $defautData['playlistVideoCount'] = count($lot->getPlaylist()->get_videos());
                $defautData['priceMin'] = $lot->getPriceMin();
                $defautData['priceMax'] = $lot->getPriceMax();
                $defautData['categorieTree'] = $this->categorieTree($lot->getCategorie());
                $defautData['gabarit'] = $lot->getGabarit();
                $defautData['nbProducts'] = $lot->getNbProduit();
                $defautData['nbJours'] = $lot->getNbJours();
            }
        }

        $this->assign([
            'playlists' => PlaylistManager::set(Wp_caller::getInstance())->get_playlists(),
            'gabarits' => GabaritManager::instance()->getGabarits(),
            'datas' => $defautData,
        ]);

        $this->setWPTemplate('Creation_video/add_lot');
    }

    private function getStats()
    {
        $waiting = Wp_caller::getInstance()->execute_video_request("SELECT count(*) as 'cpt' FROM %s WHERE `flag` = 'waiting'");
        $in_progress = Wp_caller::getInstance()->execute_video_request("SELECT count(*) as 'cpt' FROM %s WHERE `flag` = 'in_progress'");
        $error = Wp_caller::getInstance()->execute_video_request("SELECT count(*) as 'cpt' FROM %s WHERE `flag` = 'error'");

        return [
            'nb_videos_attente' => $waiting[0]['cpt'],
            'nb_videos_en_cours_creation' => $in_progress[0]['cpt'],
            'nb_video_erreur' => $error[0]['cpt'],
        ];
    }

    public function deleteDirectory($dir)
    {
        if (!file_exists($dir)) {
            return true;
        }

        if (!is_dir($dir)) {
            return unlink($dir);
        }

        foreach (scandir($dir) as $item) {
            if ($item == '.' || $item == '..') {
                continue;
            }

            if (!$this->deleteDirectory($dir . DIRECTORY_SEPARATOR . $item)) {
                return false;
            }
        }

        return rmdir($dir);
    }

    private function handleDeleteVideoAjaxRequest()
    {
        if (Tools::getIsset('id') && Tools::getIsset('delete') && Tools::getValue('delete') == 'true') {
            $id = Tools::getValue('id');

            ob_get_clean();
            ob_start();

            // delete video files
            $this->deleteDirectory(__DIR__ . '/../Videos/' . $id);

            $filesIsDeleted = !file_exists(__DIR__ . '/../Videos/' . $id);

            if ($filesIsDeleted) {
                try {
                    foreach ((new PlaylistManager(Wp_caller::getInstance()))->get_playlists() as $playlist) {
                        $needUpdate = false;
                        $videos = $playlist->get_videos();
                        if (($key = array_search($id, $videos)) !== false) {
                            $needUpdate = true;
                            unset($videos[$key]);
                        }
                        if ($needUpdate) {
                            $playlist->set_videos($videos);
                            if (!(new PlaylistManager(Wp_caller::getInstance()))->update_playlist($playlist)) {
                                echo DigitalSignageTranslate::getTraduction('Suppression_impossible');
                                exit(ob_get_clean());
                            }
                        }
                    }
                } catch (DigitalSignageException $e) {
                    echo $e->getMessage();
                }
            } else {
                echo DigitalSignageTranslate::getTraduction('Suppression_files_impossible');
                exit(ob_get_clean());
            }

            if (Wp_caller::getInstance()->execute_video_request('DELETE FROM %s WHERE `id_demande` = ' . $id) === false) {
                echo DigitalSignageTranslate::getTraduction('MAJ_bdd_impossible');
                exit(ob_get_clean());
            }

            $result = ob_get_clean();

            if (empty($result)) {
                DisplayedVideoProduct::removeDisplayedVideoProduct($id);
                exit('ok');
            }

            exit($result);
        }
    }

    /**
     * @throws DigitalSignageException
     */
    private function getAutomatisation()
    {
        $autos = AutomatisationManager::set(Wp_caller::getInstance())->get_automatisations();

        foreach ($autos as $key => $auto) {
            // Playlsit
            $playlist = PlaylistManager::set(Wp_caller::getInstance())->get_playlist($auto->getPlaylist());
            if ($playlist) {
                $autos[$key]->setPlaylist($playlist);
            } else {
                $autos[$key]->setPlaylist(null);
            }

            // Gabarit
            $gabarit = GabaritManager::instance()->getGabarit($auto->getGabarit());
            if ($gabarit) {
                $autos[$key]->setGabarit($gabarit);
            }
        }

        return $autos;
    }

    /**
     * @throws DigitalSignageException
     */
    public function generateCreationPage()
    {
        if (Tools::getIsset('save_automatisation')) {
            ob_get_clean();
            ob_start();

            if (empty($_POST['when'])) {
                $_POST['when'] = ' ';
            }

            $update = true;

            $id = Tools::getValue('id');
            if (!$id) {
                $id = PlaylistManager::set(Wp_caller::getInstance())->get_last_id() + 1;
                $update = false;
            }

            $automatisation = new AutomatisationEntity();
            $automatisation->setActive(true)->setId($id)->setName(stripslashes(Tools::getValue('name')))->setCategorie(Tools::getValue('categories') == '' ? [] : explode(',', Tools::getValue('categories')))->setCreated(0)->setHookCreation(strpos(Tools::getValue('when'), 'creat') !== false)->setHookSell(strpos(Tools::getValue('when'), 'sell') !== false)->setPriceMax(Tools::getValue('prixMax'))->setPriceMin(Tools::getValue('prixMin'))->setGabarit(Tools::getValue('gabarit'))->setPlaylist(Tools::getValue('playlist'))->setNbProduit(Tools::getValue('sellNbProduit'))->setNbJours(Tools::getValue('sellNbJours'));

            try {
                $automatisationInstance = new AutomatisationManager(Wp_caller::getInstance());

                if ($update) {
                    if ($automatisationInstance->get_automatisation($id) == $automatisation) {
                        echo DigitalSignageTranslate::getTraduction('Parametres_inchangé');
                    } else {
                        if ($automatisationInstance->update_automatisation($automatisation)) {
                            echo 'ok:' . $id;
                        } else {
                            echo DigitalSignageTranslate::getTraduction('MAJ_bdd_impossible');
                        }
                    }
                } else {
                    if ($automatisationInstance->add_automatisation($automatisation)) {
                        echo 'ok:' . $id;
                    } else {
                        echo DigitalSignageTranslate::getTraduction('MAJ_bdd_impossible');
                    }
                }
            } catch (DigitalSignageException $e) {
                echo 'erreur : ' . $e->getMessage();
            }

            exit(ob_get_clean());
        }

        $this->assign([
            'playlists' => PlaylistManager::set(Wp_caller::getInstance())->get_playlists(),
            'gabarits' => GabaritManager::instance()->getGabarits(),
            'datas' => $this->getDatas(),
        ]);

        $this->setWPTemplate('Creation_video/add_auto');
    }

    private function categorieTree($selected_cat = [])
    {
        $categories = $this->getWooCommerceCategories();
        ob_start();
        echo "<div class='caseCategorie'>";
        $this->categoriesGenerateTree($categories, $selected_cat, 0);
        echo '</div>';

        return ob_get_clean();
    }

    private function getWooCommerceCategories()
    {
        $args = [
            'taxonomy' => 'product_cat',
            'orderby' => 'name',
            'show_count' => 0,
            'pad_counts' => 0,
            'hierarchical' => 0,
            'title_li' => '',
            'hide_empty' => 0,
        ];

        return get_categories($args);
    }

    private function categoriesGenerateTree($categories, $selected_cat, $parentId)
    {
        foreach ($categories as $categorie) {
            if ($categorie->parent == $parentId) {
                $checked = (in_array($categorie->term_id, $selected_cat)) ? 'checked' : '';
                echo '<div style="margin-left: 20px"><label><input type="checkbox" name="cat[]" id="" ' . $checked . ' value="' . $categorie->term_id . '">' . $categorie->name . '</label><br>';
                $this->categoriesGenerateTree($categories, $selected_cat, $categorie->term_id);
                echo '</div>';
            }
        }
    }

    private function handleCreationAjaxRequest()
    {
        $post = $_POST;

        if (isset($post['request'])) {
            ob_get_clean();
            ob_start();

            try {
                switch ($post['request']) {
                    case 'rendu_Lot':
                        $lots_ = $this->getLot();
                        $stop_refresh = true;

                        $html = '<div class="row header">

                <div class="cell" style="width: calc(80px + 5em);">
                </div>


                <div class="cell">
                    ' . DigitalSignageTranslate::getTraduction('Nom') . '
                </div>

                <div class="cell">
                    ' . DigitalSignageTranslate::getTraduction('Video_cree') . '
                </div>

                <div class="cell">
                    Etat
                </div>

                <div class="cell">
                    ' . DigitalSignageTranslate::getTraduction('Playlist') . '
                </div>

                <div class="cell">
                    ' . DigitalSignageTranslate::getTraduction('Gabarit') . '
                </div>

            </div>';

                        foreach ($lots_ as $elt) {
                            // si un lot est en cours mettre $stop_refresh a true
                            if ($elt->getEtat() != 'Terminé') {
                                $stop_refresh = false;
                            }
                            // generer html de list lot!!
                            $html .= '<div class="row" id="row_' . $elt->getId() . '">
											<div class="cell" data-title="" style="display: flex; width: calc(80px + 5em);">
											<button style="padding: .55em; border-radius: 2em;background-color: #25b9d7; border: none " class="btn btn_ds btn-primary"  onclick="location.href=location.href+\'&traitement_par_lotVue=add&lot_id=' . $elt->getId() . '\'">
													<span class="material-icons">
														edit
													</span>
											</button>
											<button style="padding: .55em; border-radius: 2em; background-color: #e07281; border: none; margin-left: 1em !important;" onclick="saveIdDelete(' . $elt->getId() . ')" class="btn btn_ds btn-primary" data-toggle="modal" data-target="#Modal">
													 <span class="material-icons">
														delete
													</span>
											</button>
										</div>
										<div class="cell" data-title="' . DigitalSignageTranslate::getTraduction('Nom') . '">
											' . $elt->getName() . '
										</div>
										<div class="cell" data-title="' . DigitalSignageTranslate::getTraduction('Video_cree') . '">
											';
                            if ($elt->getCreated()) {
                                $html .= $elt->getCreated();
                            } else {
                                $html .= '0';
                            }

                            $html .= '</div>
										<div class="cell" data-title="Etat">
											' . $elt->getEtat() . '
										</div>
										<div class="cell" data-title="' . DigitalSignageTranslate::getTraduction('Playlist') . '">';

                            if ($elt->getPlaylist() != '') {
                                $html .= '<a href="' . Tools_DS4::getUrlRedirect('DigitalSignagePlaylists') . '&playlistVue=update&id_playlist=' . $elt->getPlaylist()->get_id() . '">' . $elt->getPlaylist()->get_title() . '</a>';
                            }

                            $html .= '</div>
										<div class="cell" data-title="' . DigitalSignageTranslate::getTraduction('Gabarit') . '">
											' . $elt->getGabarit()->getNom() . '
										</div>
									</div>';
                        }

                        echo json_encode(['html' => $html, 'stop_refresh' => $stop_refresh]);

                        break;
                    case 'calcul_nb_pdt_Lot':
                        // utiliser les param post pour determiner le nombre de video qui vont être générées
                        $Lot_ = LotManager::set(Wp_caller::getInstance());
                        $liste_product = $Lot_->get_products($post['categories'], $post['prixMin'], $post['prixMax']);

                        echo json_encode(['nb' => count($liste_product)]);

                        break;
                    case 'changeActive':
                        $id = $post['automatisationId'];
                        $active = $post['active'];
                        $autoc = AutomatisationManager::set(Wp_caller::getInstance());
                        $automat = $autoc->get_automatisation($id);

                        if ($automat) {
                            $automat->setActive($active);

                            if ($autoc->update_automatisation($automat)) {
                                echo 'ok';
                            } else {
                                echo 'error';
                            }
                        } else {
                            echo DigitalSignageTranslate::getTraduction('Erreur_automat_non_trouve');
                        }

                        break;

                    case 'deleteAuto':
                        if (AutomatisationManager::set(Wp_caller::getInstance())->remove_automatisation(AutomatisationManager::set(Wp_caller::getInstance())->get_automatisation($post['id']))) {
                            echo 'ok';
                        } else {
                            echo DigitalSignageTranslate::getTraduction('Suppression_impossible');
                        }

                        break;

                    case 'deleteLot':
                        if (LotManager::set(Wp_caller::getInstance())->remove_lot(LotManager::set(Wp_caller::getInstance())->get_lot($post['id']))) {
                            echo 'ok';
                        } else {
                            echo DigitalSignageTranslate::getTraduction('Suppression_impossible');
                        }

                        break;

                    case 'add_playlist':
                        $playlist_id = null;
                        $playlist_name = null;

                        if (!empty($_POST['new_playlist_name'])) {
                            $playlist_controller = new PlaylistManager(Wp_caller::getInstance());
                            $playlist_id = $playlist_controller->get_last_id() + 1;
                            $playlist_name = stripslashes(Tools::getValue('new_playlist_name'));

                            $playlist = new PlaylistEntity();
                            $playlist->set_videos([])->set_title($playlist_name)->set_active(true)->set_limit(0)->set_id($playlist_id);

                            try {
                                if (!$playlist_controller->add_playlist($playlist)) {
                                    $playlist_id = null;
                                    $playlist_name = null;
                                }
                            } catch (DigitalSignageException $e) {
                                $playlist_id = null;
                                $playlist_name = null;
                            }
                        }

                        echo json_encode([$playlist_name, $playlist_id]);

                        break;
                }
            } catch (DigitalSignageException $e) {
                $e->echoError();
            }

            exit(ob_get_clean());
        }
    }

    private function getDatas()
    {
        $defautData = [
            'id' => '',
            'name' => '',
            'playlist' => '',
            'populaire' => false,
            'new' => false,
            'priceMin' => '',
            'priceMax' => '',
            'categorieTree' => $this->categorieTree(),
            'gabarit' => '',
            'nbProducts' => '5',
            'nbJours' => '5',
        ];

        if (Tools::getIsset('auto_id')) {
            $automatisation = AutomatisationManager::set(Wp_caller::getInstance())->get_automatisation(Tools::getValue('auto_id'));
            if ($automatisation) {
                $defautData['id'] = $automatisation->getId();
                $defautData['name'] = $automatisation->getName();
                $defautData['playlist'] = $automatisation->getPlaylist();
                $defautData['populaire'] = $automatisation->getHookSell();
                $defautData['new'] = $automatisation->getHookCreation();
                $defautData['priceMin'] = $automatisation->getPriceMin();
                $defautData['priceMax'] = $automatisation->getPriceMax();
                $defautData['categorieTree'] = $this->categorieTree($automatisation->getCategorie());
                $defautData['gabarit'] = $automatisation->getGabarit();
                $defautData['nbProducts'] = $automatisation->getNbProduit();
                $defautData['nbJours'] = $automatisation->getNbJours();
            }
        }

        return $defautData;
    }
}
