<?php
/**
 * @author Acacia Dynamic
 * @copyright 2024 Acacia Dynamic
 * @license private
 */
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

if (file_exists('./../../../../../wp-load.php')) {
    define('WP_USE_THEMES', false); // Don't load theme support functionality
    require './../../../../../wp-load.php';
} else {
    require './../../../../../config/config.inc.php';
    require './../../../../../init.php';
    require_once './../../../digitalsignage.php';
    DigitalSignageTranslate::setLang(substr(get_bloginfo('language'), 0, 2));
    refresh_local_token();
}

require_once __DIR__ . '/../../Kernel/Translations/DigitalSignageTranslate.php';
require_once __DIR__ . '/../../Classes/Wp_caller.php';
require_once __DIR__ . '/../../controllers/VideoController.php';
require_once __DIR__ . '/../../Kernel/API/Entity/DemandeEntity.php';
require_once __DIR__ . '/../../Kernel/API/Manager/DemandeManager.php';
require_once __DIR__ . '/../../Kernel/API/Manager/DigitalSignageException.php';
require_once __DIR__ . '/../../Kernel/API/Manager/GabaritManager.php';
require_once __DIR__ . '/../../Classes/CreationVideoSpecialDataProvider.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

new AddVideoHandler();

class AddVideoHandler
{
    private $responde = [
        'statut' => 'ok',
        'error' => [],
        'data' => [],
    ];

    public function __construct()
    {
        $this->initContent();
    }

    public function initContent()
    {
        $datas = $_POST;

        if ($this->gabaritReachable($datas)) {
            $gabaritChamps = $this->getGabaritChamps($datas['gabarit']);

            $haveRequiredChampEmpty = false;

            foreach ($gabaritChamps as $champ) {
                // $isHideChamp = strval($champ['hide'] ) == "1";
                $isRequiredChamp = (string) $champ['canBeNull'] == '0';
                $champName = $champ['name'];
                $champLabel = $champ['label'];
                if ($isRequiredChamp) {
                    if (!isset($datas[$champName]) || $datas[$champName] === '') {
                        $haveRequiredChampEmpty = true;
                        $this->responde['error'][] = DigitalSignageTranslate::getTraduction('Le_champ_ne_peut_pas_etre_vide', $champLabel);
                        $this->responde['data'][] = $champName;
                    }
                }
            }

            if ($haveRequiredChampEmpty) {
                $this->responde['statut'] = 'ko';
                exit(json_encode($this->responde));
            }

            $video = new DemandeEntity();

            $demandeManager = new DemandeManager(Wp_caller::getInstance());

            if (isset($datas['produit_libelle'])) {
                $videoName = $datas['produit_libelle'];
            } elseif (isset($datas['product_id'])) {
                $videoName = $datas['product_id'];
            } else {
                $videoName = DigitalSignageTranslate::getTraduction('Default_video_name');
            }

            $video->setData($datas);
            $video->setGabarit($datas['gabarit']);
            $video->setDateBegin(date('y-m-d H:i:s'));
            $video->setDateEnd(date('y-m-d H:i:s'));
            $video->setVideoComment('');
            $video->setVideoName($videoName);
            $video->setAutomatique('0');
            $video->setDateDemande(date('Y-m-d H:i:s'));
            $video->setProductId($datas['product_id']);
            $video->setDownloaded('0');

            $specialChampsManager = new CreationVideoSpecialDataProvider();

            $data_youtube = $specialChampsManager->getyoutubevideo($datas['product_id']);
            $video->setYoutubeData(json_encode($data_youtube));

            // poste la création de video
            try {
                $demande = $demandeManager->postDemande($video);

                if ($demande) {
                    //
                    //						dump("INSERT INTO %s (`id_demande`, `data`,                    `flag`,                    `video_name`,                   `video_comment`,                  `date_begin`,                   `automatique`,                     `date_end`,                   `date_demande`,            `gabarit`,                    `video_url`,                   `video_hash`,                  `product_id`) VALUES
                    //                                                    (NULL,        '".$demande->getData()."', '".$demande->getFlag()."', '".$demande->getVideoName()."', '".$demande->getVideoComment()."', '".$demande->getDateBegin()."', '".$demande->getAutomatique()."', '".$demande->getDateEnd()."', '".$demande->getDateDemande()."', '".$demande->getGabarit()."', '".$demande->getVideoUrl()."', '".$demande->getVideoHash()."', '".$demande->getProductId()."') ");

                    $id = Wp_caller::getInstance()->execute_video_insert(
                        'INSERT INTO `%s` ( `id_demande` 							, `data`,								                    `flag`,                          `video_name`,                                                  `video_comment`,                                                 `date_begin`,                   `automatique`,                     `date_end`,                               `date_demande`,                          `gabarit`,                    `video_url`,                   `video_hash`,`product_id`,`youtube_data`) VALUES
                                                ( ' . $demande->getIdDemande() . ' ,        "' . pSQL($demande->getData()) . '", "' . $demande->getFlag() . '", "' . pSQL($demande->getVideoName()) . '", "' . pSQL($demande->getVideoComment()) . '", "' . $demande->getDateBegin() . '", "' . $demande->getAutomatique() . '", "' . $demande->getDateEnd() . '", "' . $demande->getDateDemande() . '", "' . $demande->getGabarit() . '", "' . $demande->getVideoUrl() . '", "' . $demande->getVideoHash() . '", "' . $demande->getProductId() . '", "' . pSQL($demande->getYoutubeData()) . '") '
                    );

                    if ($id) {
                        $this->responde['statut'] = 'ok';
                        $this->responde['data'] = $id;
                    } else {
                        $this->responde['statut'] = 'ko';
                        $this->responde['error'][] = DigitalSignageTranslate::getTraduction('Enregistrement_video_bdd_impossible');
                    }
                } else {
                    $this->responde['statut'] = 'ko';
                    $this->responde['error'] = array_merge($this->responde['error'], $demandeManager->getUserMessages());
                }
            } catch (DigitalSignageException $e) {
                $this->responde['error'][] = $e->getMessage();
            }
        } else {
            $this->responde['error'][] = DigitalSignageTranslate::getTraduction('choix_gabarit_vide');
            $this->responde['statut'] = 'ko';
        }
        exit(json_encode($this->responde));
    }

    /**
     * Verifie que le gabarit n'est pas vide
     *
     * @param array $datas
     *
     * @return bool
     */
    private function gabaritReachable(array $datas)
    {
        if (isset($datas['gabarit'])) {
            if ($datas['gabarit'] != 'undefined' && !empty($datas['gabarit'])) {
                return true;
            } else {
                $this->responde['error'][] = DigitalSignageTranslate::getTraduction('Gabarit_introuvable');

                return false;
            }
        } else {
            return false;
        }
    }

    private function getGabaritChamps($gabarit)
    {
        $gabaritManager = new GabaritManager();

        $gabarit = $gabaritManager->getGabarit($gabarit);

        return $gabarit->getChamps();
    }
}

echo 'fin';
