<?php
/**
 * @author Acacia Dynamic
 * @copyright 2024 Acacia Dynamic
 * @license private
 */
require_once __DIR__ . '/../../Classes/Tools_DS4.php';
require_once __DIR__ . '/../../Classes/Wp_caller.php';
require_once __DIR__ . '/../../controllers/DigitalSignageController.php';
require_once __DIR__ . '/../../controllers/override/UpdateVideoHandleManager.php';
require_once __DIR__ . '/../../Kernel/API/Manager/DemandeManager.php';
require_once __DIR__ . '/../../Kernel/API/Manager/ForfaitManager.php';
require_once __DIR__ . '/../../Kernel/API/Manager/GabaritManager.php';
require_once __DIR__ . '/../../controllers/VideoController.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class wooCommerceProductOverride extends DigitalSignageController
{
    private $params = [];

    private $idVideo = -1;

    private $article;

    private $productDatas = [];

    private $productId = '';

    private $productImgs = [];

    /**
     * @param WC_Product $article
     */
    public function setArticle($article)
    {
        $productTemp = new WC_Product($article->get_id());
        $this->article = $article;
        $this->productDatas = $productTemp->get_data();
        $this->productDatas['permalink'] = $productTemp->get_permalink();

        $terms = get_the_terms($article->get_id(), 'product_cat');
        if ($terms && !is_wp_error($terms)) {
            $this->productDatas['first_category'] = $terms[0]->name;
        } else {
            $this->productDatas['first_category'] = '';
        }

        $this->productDatas['pourcent_reduc'] = '';
        if ($this->productDatas['regular_price'] == $this->productDatas['price']) {
            $this->productDatas['pourcent_reduc'] = '0%';
        } else {
            if ($this->productDatas['regular_price'] > 0) {
                $this->productDatas['pourcent_reduc'] = '-' . round(($this->productDatas['price'] * 100) / $this->productDatas['regular_price']) . '%';
            }
        }

        $this->productId = $productTemp->get_id();
        $this->productImgs = $this->getImages($this->productId);
    }

    public function generatePage()
    {
        if (!$this->article) {
            return false;
        }

        // TODO
        $gabarits = GabaritManager::instance()->getGabarits();

        foreach ($gabarits as $gabarit_) {
            $gabarit_->initCustomChamp();
        }

        $this->assign([
            'product' => $this->productDatas,
            'product_img' => $this->productImgs,
            'urlSpecialData' => Tools_DS4::getModuleUrlPresta() . 'ds4woo/Classes/CreationVideoSpecialDataProviderAjaxHandler.php?digitalSignageSpecialData=',
            'urlChangeDisplayedVideo' => Tools_DS4::getModuleUrlPresta() . 'ds4woo/Classes/DisplayedVideoProductAjaxHandler.php',
            'urlLoadingImg' => Tools_DS4::getModuleImgDirPresta() . 'loadingChamp.gif',
            'gabarits' => $gabarits,
            'urlTraitement' => plugin_dir_url(__FILE__) . 'AddVideoHandler.php',
            'urlVideos' => 'admin.php?page=DigitalSignageVideos',
            'videoPath' => Tools_DS4::getModuleVideoUrl(),
            'updateController' => plugin_dir_url(__FILE__) . 'UpdateVideoHandler.php',
            'user_connected' => ForfaitManager::instance()->getToken() !== null,
            'currentHistoriqueHtml' => $this->getHistoriqueHtml(),
            // 'NbrActiveVideo'		  => ForfaitManager::instance()->getForfait()->getActiveVideos() - (new VideoController())->getNbrVideosActives(),
            'NbrActiveVideo' => 0,
        ]);

        $this->setWPTemplate('../hook/front/adminProductCreateVideoForm');

        return true;
    }

    public function getHistoriqueHtml()
    {
        $contentHtml = '';

        try {
            $videoForThisProduct = DemandeManager::set(Wp_caller::getInstance())->getLocalDemandes('SELECT * FROM %s WHERE `product_id` = ' . $this->productId);
        } catch (Exception $e) {
            $videoForThisProduct = [];
        }

        foreach ($videoForThisProduct as $video) {
            $videoHistoriqueManager = new UpdateVideoHandlerManager(true, $video->getIdDemande(), $video->getProductId());
            $contentHtml = $contentHtml . $videoHistoriqueManager->getHtmlDatas();
        }

        return $contentHtml;
    }

    public function getImages($id_product)
    {
        $images = [];
        $miniature = wp_get_attachment_image_src(get_post_thumbnail_id($id_product), 'single-post-thumbnail');
        if ($miniature === false) {
            return $images;
        }
        array_push($images, $miniature[0]);
        $attachment_ids = $this->article->get_gallery_image_ids();
        array_map(
            function ($di_image) use (&$images) {
                array_push($images, wp_get_attachment_url($di_image));
            },
            $attachment_ids
        );

        return array_map(
            function ($image) {
                $uploads = wp_upload_dir();
                $file_path = str_replace($uploads['baseurl'], $uploads['basedir'], $image);

                return [
                    'url' => $image,
                    'path' => $file_path,
                ];
            },
            $images
        );
    }
}
