<?php
/**
 * @author Acacia Dynamic
 * @copyright 2024 Acacia Dynamic
 * @license private
 */
// session_start();

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

/**
 * Plugin Name: Digital Signage 4
 * Plugin URI: https://www.acacia-dynamic.com/
 * Version: 1.0
 * Description: Sell more of your products with quality videos created automatically from all the elements of your product sheets. Create in one click videos adapted to events: Sales, promotions, Valentine's Day, Christmas, Black Friday, ...
 * Author: Acacia Dynamic
 * Author URI: https://www.acacia-dynamic.com/
 * Text Domain: digitalsignage
 **/

require_once __DIR__ . '/Classes/FrontShortcodeVideoDisplayer.php';
require_once __DIR__ . '/Classes/Wp_caller.php';
require_once __DIR__ . '/controllers/hook/AutomatisationHookControllerCreation.php';
require_once __DIR__ . '/controllers/override/wooCommerceProductOverride.php';
require_once __DIR__ . '/controllers/ParametreController.php';
require_once __DIR__ . '/controllers/PlaylistsController.php';
require_once __DIR__ . '/controllers/PresentationController.php';
require_once __DIR__ . '/controllers/VideoController.php';
require_once __DIR__ . '/controllers/GabaritController.php';
require_once __DIR__ . '/Kernel/API/Manager/DigitalSignageException.php';
require_once __DIR__ . '/Kernel/API/Manager/ForfaitManager.php';
require_once __DIR__ . '/Kernel/Translations/DigitalSignageTranslate.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

// Point d'entrée du module
new digitalsignage();

// Class d'entrée du module, inclusion des hooks
class digitalsignage
{
    private static $userAliveAlreadyChecked = false;

    /**
     * digitalsignage constructor.
     */
    public function __construct()
    {
        DigitalSignageTranslate::setLang(substr(get_bloginfo('language'), 0, 2));

        // permet de traduire la description du module qui est en commentaire
        load_plugin_textdomain(
            'digitalsignage',
            false,
            'digitalsignage/lang/'
        );

        FrontShortcodeVideoDisplayer::init();

        // tabs
        add_action('admin_menu', [$this, 'add_digitalsignage_tabs']); // Ajout du hooks des tabs du module

        // styles and scripts
        add_action('admin_init', [$this, 'setModuleBackofficeStyle']);
        add_action('admin_init', [$this, 'setModuleBackOfficeScripts']);

        // custom vidéo database
        add_action('admin_init', [$this, 'installCustomDbVideo']);

        // clear le buffer
        add_action('init', [$this, 'init']);

        // meta box dans les pages d'articles
        add_action('add_meta_boxes', [$this, 'digital_signage_metabox']);

        // event sur le changement d'etat d'un article
        add_action('transition_post_status', [
            new AutomatisationHookControllerCreation(),
            'handleProductChangeStatus',
        ], 9999, 3);

        $this->user_connection_alive();
    }

    public function setModuleBackofficeStyle()
    {
        wp_enqueue_style('bootstrapcss', 'https://stackpath.bootstrapcdn.com/bootstrap/4.5.2/css/bootstrap.min.css', [], false, 'all');
        wp_enqueue_style('bootstrapcss');
        wp_register_style('digitalsignage_css', plugins_url('/views/css/styles.css', __FILE__));
        wp_enqueue_style('digitalsignage_css');
    }

    public function setModuleBackOfficeScripts()
    {
        wp_register_script('bootstrapjs', 'https://stackpath.bootstrapcdn.com/bootstrap/4.5.2/js/bootstrap.min.js', ['jquery'], false, true);
        wp_enqueue_script('bootstrapjs');
        wp_register_script('digitalsignage_script', plugins_url('/views/js/script.js', __FILE__), [], false, true);
        wp_enqueue_script('digitalsignage_script');
    }

    // Ajout des tabs du module au menu admin
    public function add_digitalsignage_tabs()
    {
        $tabs = [
            [
                'name' => DigitalSignageTranslate::getTraduction('Digital_Signage_4'),
                'slug' => 'DigitalSignageHome',
                'capability' => 'manage_options',
                'callback' => '',
                'icon' => 'dashicons-format-video',
                'position' => 6,
            ],
            [
                'parent_slug' => 'DigitalSignageHome',
                'name' => DigitalSignageTranslate::getTraduction('Accueil'),
                'slug' => 'DigitalSignageHome',
                'capability' => 'manage_options',
                'callback' => [new PresentationController(), 'generatePage'],
                'position' => null,
            ],
            [
                'parent_slug' => 'DigitalSignageHome',
                'name' => DigitalSignageTranslate::getTraduction('Videos'),
                'slug' => 'DigitalSignageVideos',
                'capability' => 'manage_options',
                'callback' => [new VideoController(), 'generatePage'],
                'position' => null,
            ],
            [
                'parent_slug' => 'DigitalSignageHome',
                'name' => DigitalSignageTranslate::getTraduction('Playlists'),
                'slug' => 'DigitalSignagePlaylists',
                'capability' => 'manage_options',
                'callback' => [new PlaylistsController(), 'generatePage'],
                'position' => null,
            ],
            [
                'parent_slug' => 'DigitalSignageHome',
                'name' => 'Modèles',
                'slug' => 'DigitalSignagePersoGabarit',
                'capability' => 'manage_options',
                'callback' => [new GabaritController(), 'generatePage'],
                'position' => null,
            ],
            [
                'parent_slug' => 'DigitalSignageHome',
                'name' => DigitalSignageTranslate::getTraduction('Reglages'),
                'slug' => 'DigitalSignageSettings',
                'capability' => 'manage_options',
                'callback' => [new ParametreController(), 'generatePage'],
                'position' => null,
            ],
        ];

        foreach ($tabs as $tab) {
            if (isset($tab['parent_slug'])) {
                add_submenu_page($tab['parent_slug'], $tab['name'], $tab['name'], $tab['capability'], $tab['slug'], $tab['callback'], $tab['position']);
            } else {
                add_menu_page($tab['name'], $tab['name'], $tab['capability'], $tab['slug'], $tab['callback'], $tab['icon'], $tab['position']);
            }
        }
    }

    // Permet la redirection via php en bloquant les affichage d'html avant la redirection
    public function init()
    {
        wp_register_script('digitalsignage_script_front', plugins_url('/views/js/digitalsignage.js', __FILE__), [], false, true);
        wp_enqueue_script('digitalsignage_script_front');
        ob_start();
    }

    public function installCustomDbVideo()
    {
        if (!Wp_caller::getInstance()->create_video_table()) {
            Context::getContext()->cookie->errors = DigitalSignageTranslate::getTraduction('Creation_bdd_impossible');
        }
    }

    public function digital_signage_metabox()
    {
        $screen = 'product';
        if (wc_get_product()) {
            add_meta_box(
                'digital_signage',                 // Unique ID
                DigitalSignageTranslate::getTraduction('Digital_Signage_4_creation'),      // Box title
                [$this, 'wporg_custom_box_html'],  // Content callback, must be of type callable
                $screen                            // Post type
            );
        }
    }

    public function wporg_custom_box_html()
    {
        $controller = new wooCommerceProductOverride();
        $controller->setArticle(wc_get_product());
        $controller->generatePage();
    }

    private function user_connection_alive()
    {
        $forfait = new ForfaitEntity();
        $success = $forfait->fromArray(Wp_caller::getInstance()->get_data(Wp_caller::FORFAIT));

        if ($success) {
            ForfaitManager::instance()->setForfait($forfait);
        } elseif (!self::$userAliveAlreadyChecked) { // Si aucun forfait n'existe alors on créer un compte de test pour l'utilisateur
            try {
                $forfait = ForfaitManager::instance()->createForfaitTrial();

                if (!$forfait) {
                    foreach (ForfaitManager::instance()->getUserMessages() as $error) {
                        Context::getContext()->cookie->errors = $error;
                    }
                }
            } catch (DigitalSignageException $e) {
                $e->echoError();
            }
            self::$userAliveAlreadyChecked = true;
        }
    }
}
